# -*- coding: utf-8 -*-
# Author: TDC Team
# License: MIT
import os

from .base_group import BenchmarkGroup


class SCDTIGroup(BenchmarkGroup):
    """Create SCDTI Group Class object. This is for single-cell drug-target identification task benchmark.

    Args:
            path (str, optional): the path to store/retrieve the SCDTI group datasets.
    """

    def __init__(self, path="./data", file_format="csv"):
        """Create an SCDTI benchmark group class."""
        # super().__init__(name="SCDTI_Group", path=path)
        self.name = "SCDTI_Group"
        self.path = os.path.join(path, self.name)
        # self.datasets = ["opentargets_dti"]
        self.dataset_names = ["opentargets_dti"]
        self.file_format = file_format
        self.split = None

    def get_train_valid_split(self):
        """parameters included for compatibility. this benchmark has a fixed train/test split."""
        from ..resource.dataloader import DataLoader
        if self.split is None:
            dl = DataLoader(name="opentargets_dti")
            self.split = dl.get_split()
        return self.split["train"], self.split["dev"]

    def get_test(self):
        from ..resource.dataloader import DataLoader
        if self.split is None:
            dl = DataLoader(name="opentargets_dti")
            self.split = dl.get_split()
        return self.split["test"]

    def evaluate(self, y_pred):
        from sklearn.metrics import precision_score, recall_score, accuracy_score, f1_score
        y_true = self.get_test()["Y"]
        # Calculate metrics
        precision = precision_score(y_true, y_pred)
        recall = recall_score(y_true, y_pred)
        accuracy = accuracy_score(y_true, y_pred)
        f1 = f1_score(y_true, y_pred)
        return [precision, recall, accuracy, f1]

    def evaluate_many(self, preds):
        from numpy import mean, std
        if len(preds) < 5:
            raise Exception(
                "Run your model on at least 5 seeds to compare results and provide your outputs in preds."
            )
        out = dict()
        preds = [self.evaluate(p) for p in preds]
        out["precision"] = (mean([x[0] for x in preds]),
                            std([x[0] for x in preds]))
        out["recall"] = (mean([x[1] for x in preds]), std([x[1] for x in preds
                                                          ]))
        out["accuracy"] = (mean([x[2] for x in preds]),
                           std([x[2] for x in preds]))
        out["f1"] = (mean([x[3] for x in preds]), std([x[3] for x in preds]))
        return out



from ..utils import general_load
from ..utils.load import download_wrapper, load_json_from_txt_file

import pandas as pd
import os
import torch


class PINNACLE:
    """
    PINNACLE is a class for loading and manipulating the PINNACLE networks and embeddings.
    @article{
        Li2023,
        author = "Michelle Li",
        title = "{PINNACLE}",
        year = "2023",
        month = "4",
        url = "https://figshare.com/articles/software/AWARE/22708126",
        doi = "10.6084/m9.figshare.22708126.v5"
    }
    """

    def __init__(self, path="./data"):
        self.ppi_name = "pinnacle_global_ppi_edgelist"
        self.cell_tissue_mg_name = "cell_tissue_mg_edgelist"
        self.ppi = general_load(self.ppi_name, path, " ")
        self.ppi.columns = ["Protein A", "Protein B"]
        self.cell_tissue_mg = general_load(
            self.cell_tissue_mg_name, path,
            "\t")  # use tab as names were left with spaces
        self.cell_tissue_mg.columns = ["Tissue", "Cell"]
        self.embeds_name = "pinnacle_protein_embed"
        # self.embeds = resource_dataset_load(self.embeds_name, path, [self.embeds_name])
        self.embeds_name = download_wrapper(self.embeds_name, path,
                                            self.embeds_name)
        self.embeds = torch.load(os.path.join(path, self.embeds_name + ".pth"))
        self.keys = load_json_from_txt_file("pinnacle_labels_dict", path)

    def get_ppi(self):
        return self.ppi

    def get_mg(self):
        return self.cell_tissue_mg

    def get_embeds_raw(self):
        return self.embeds

    def get_keys(self):
        protein_names_celltypes = [
            p for p in zip(self.keys["Cell Type"], self.keys["Name"])
            if not (p[0].startswith("BTO") or p[0].startswith("CCI") or
                    p[0].startswith("Sanity"))
        ]
        proteins = pd.DataFrame.from_dict({
            "target": [n for _, n in protein_names_celltypes],
            "cell type": [c for c, _ in protein_names_celltypes]
        })
        proteins.drop_duplicates()
        return proteins

    def get_embeds(self):
        prots = self.get_keys()
        emb = self.get_embeds_raw()
        # nemb = {'--'.join(prots.iloc[k]): v for k, v in emb.items()}
        x = {}
        ctr = 0
        for _, v in emb.items():
            if isinstance(v, torch.Tensor):
                if v.size()[0] == 1:
                    k = "--".join(prots.iloc[ctr])
                    ctr += 1
                    x[k] = v.detach().numpy()
                    continue
                for t in v:
                    assert len(t.size()) == 1, t.size()
                    k = "--".join(prots.iloc[ctr])
                    ctr += 1
                    x[k] = t.detach().numpy()
            else:
                raise Exception("encountered non-tensor")
        assert len(x) == len(prots), "dict len {} vs keys length {}".format(
            len(x), len(prots))
        df = pd.DataFrame.from_dict(x)
        df = df.transpose()
        assert len(df) == len(
            x), "dims not mantained when translated to pandas. {} vs {}".format(
                len(df), len(x))
        return df
